# Generated by roxygen2: do not edit by hand

S3method("[",phyDat)
S3method("[",splits)
S3method(AICc,pml)
S3method(BIC,pml)
S3method(addConfidences,multiPhylo)
S3method(addConfidences,networx)
S3method(addConfidences,phylo)
S3method(addConfidences,splits)
S3method(anova,pml)
S3method(as.AAbin,phyDat)
S3method(as.DNAbin,phyDat)
S3method(as.Matrix,splits)
S3method(as.MultipleAlignment,phyDat)
S3method(as.StringSet,phyDat)
S3method(as.bitsplits,splits)
S3method(as.character,phyDat)
S3method(as.data.frame,ancestral)
S3method(as.data.frame,phyDat)
S3method(as.matrix,splits)
S3method(as.networx,phylo)
S3method(as.networx,splits)
S3method(as.phyDat,AAStringSet)
S3method(as.phyDat,AAbin)
S3method(as.phyDat,DNAStringSet)
S3method(as.phyDat,DNAbin)
S3method(as.phyDat,MultipleAlignment)
S3method(as.phyDat,alignment)
S3method(as.phyDat,ancestral)
S3method(as.phyDat,character)
S3method(as.phyDat,data.frame)
S3method(as.phyDat,factor)
S3method(as.phyDat,matrix)
S3method(as.phylo,splits)
S3method(as.pml,modelTest)
S3method(as.prop.part,splits)
S3method(as.splits,bitsplits)
S3method(as.splits,multiPhylo)
S3method(as.splits,networx)
S3method(as.splits,phylo)
S3method(as.splits,prop.part)
S3method(c,phyDat)
S3method(c,splits)
S3method(cbind,phyDat)
S3method(cophenetic,networx)
S3method(cophenetic,splits)
S3method(glance,phyDat)
S3method(glance,pml)
S3method(glance,pmlMix)
S3method(hash,multiPhylo)
S3method(hash,phylo)
S3method(identify,networx)
S3method(image,phyDat)
S3method(logLik,pml)
S3method(logLik,pmlMix)
S3method(logLik,pmlPart)
S3method(midpoint,multiPhylo)
S3method(midpoint,phylo)
S3method(plot,codonTest)
S3method(plot,networx)
S3method(plot,pml)
S3method(plot,pmlCluster)
S3method(plot,pmlPart)
S3method(print,SOWH)
S3method(print,ancestral)
S3method(print,codonTest)
S3method(print,phyDat)
S3method(print,pml)
S3method(print,pmlMix)
S3method(print,pmlPart)
S3method(print,splits)
S3method(print,summary.clanistics)
S3method(rbind,phyDat)
S3method(reorder,networx)
S3method(simSeq,phylo)
S3method(simSeq,pml)
S3method(subset,phyDat)
S3method(summary,SOWH)
S3method(summary,clanistics)
S3method(tidy,modelTest)
S3method(unique,dist)
S3method(unique,phyDat)
S3method(unique,splits)
S3method(update,pml)
S3method(vcov,pml)
export(AICc)
export(Ancestors)
export(CI)
export(Children)
export(Descendants)
export(KF.dist)
export(NJ)
export(RF.dist)
export(RI)
export(SH.test)
export(SOWH.test)
export(SPR.dist)
export(Siblings)
export(UNJ)
export(acctran)
export(acgt2ry)
export(add.tips)
export(addConfidences)
export(addTrivialSplits)
export(add_boxplot)
export(add_ci)
export(add_edge_length)
export(add_mutations)
export(add_support)
export(allCircularSplits)
export(allCompat)
export(allDescendants)
export(allSitePattern)
export(allSplits)
export(allTrees)
export(anc_pars)
export(anc_pml)
export(ancestral.pars)
export(ancestral.pml)
export(as.Matrix)
export(as.MultipleAlignment)
export(as.StringSet)
export(as.ancestral)
export(as.networx)
export(as.phyDat)
export(as.pml)
export(as.splits)
export(bab)
export(baseFreq)
export(bootstrap.phyDat)
export(bootstrap.pml)
export(c.phyDat)
export(candidate_tree)
export(cbind.phyDat)
export(cladePar)
export(coalSpeciesTree)
export(codon2dna)
export(codonTest)
export(compatible)
export(composition_test)
export(consensusNet)
export(coords)
export(createLabel)
export(delta.score)
export(densiTree)
export(designSplits)
export(designTree)
export(dfactorial)
export(discrete.beta)
export(discrete.gamma)
export(dist.hamming)
export(dist.logDet)
export(dist.ml)
export(dist.p)
export(distanceHadamard)
export(distinct.splits)
export(diversity)
export(dna2aa)
export(dna2codon)
export(edQt)
export(fhm)
export(fitch)
export(gap_as_ambiguous)
export(gap_as_state)
export(genlight2phyDat)
export(getClans)
export(getClips)
export(getDiversity)
export(getRoot)
export(getSlices)
export(glance)
export(h2st)
export(h4st)
export(hadamard)
export(has_gap_state)
export(hash)
export(keep_as_tip)
export(latag2n.phyDat)
export(ldfactorial)
export(lento)
export(lli)
export(map_duplicates)
export(mast)
export(matchSplits)
export(maxCladeCred)
export(mcc)
export(midpoint)
export(modelTest)
export(mrca.phylo)
export(multiphyDat2pmlPart)
export(neighborNet)
export(nni)
export(nnls.networx)
export(nnls.phylo)
export(nnls.splits)
export(nnls.tree)
export(optim.parsimony)
export(optim.pml)
export(pace)
export(parsimony)
export(path.dist)
export(phyDat)
export(phyDat2MultipleAlignment)
export(phyDat2alignment)
export(plotAnc)
export(plotBS)
export(plotRates)
export(plotSeqLogo)
export(plot_gamma_plus_inv)
export(pml)
export(pml.control)
export(pml.fit)
export(pml.free)
export(pml.init)
export(pmlCluster)
export(pmlMix)
export(pmlPart)
export(pmlPart2multiPhylo)
export(pmlPen)
export(pml_bb)
export(pratchet)
export(presenceAbsence)
export(pruneTree)
export(rNNI)
export(rSPR)
export(random.addition)
export(ratchet.control)
export(read.nexus.dist)
export(read.nexus.networx)
export(read.nexus.partitions)
export(read.nexus.splits)
export(read.phyDat)
export(readDist)
export(removeAmbiguousSites)
export(removeTrivialSplits)
export(removeUndeterminedSites)
export(sankoff)
export(simSeq)
export(splitsNetwork)
export(sprdist)
export(superTree)
export(supgma)
export(threshStateC)
export(tidy)
export(transferBootstrap)
export(treedist)
export(upgma)
export(wRF.dist)
export(wpgma)
export(write.ancestral)
export(write.nexus.dist)
export(write.nexus.networx)
export(write.nexus.splits)
export(write.phyDat)
export(write.pml)
export(write.splits)
export(writeDist)
import(Rcpp)
import(ape)
import(methods)
import(parallel)
importFrom(Matrix,Matrix)
importFrom(Matrix,crossprod)
importFrom(Matrix,solve)
importFrom(Matrix,spMatrix)
importFrom(Matrix,sparseMatrix)
importFrom(fastmatch,fmatch)
importFrom(generics,glance)
importFrom(generics,tidy)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,bxp)
importFrom(graphics,curve)
importFrom(graphics,hist)
importFrom(graphics,identify)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,locator)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.default)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,rug)
importFrom(graphics,segments)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(igraph,E)
importFrom(igraph,all_shortest_paths)
importFrom(igraph,decompose)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,layout_nicely)
importFrom(igraph,make_graph)
importFrom(igraph,shortest_paths)
importFrom(igraph,topo_sort)
importFrom(igraph,vcount)
importFrom(quadprog,solve.QP)
importFrom(quadprog,solve.QP.compact)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,binomial)
importFrom(stats,chisq.test)
importFrom(stats,constrOptim)
importFrom(stats,cophenetic)
importFrom(stats,cor)
importFrom(stats,dbinom)
importFrom(stats,dgamma)
importFrom(stats,ecdf)
importFrom(stats,hclust)
importFrom(stats,lm.fit)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,pbeta)
importFrom(stats,pchisq)
importFrom(stats,pgamma)
importFrom(stats,qbeta)
importFrom(stats,qgamma)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,reshape)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,stepfun)
importFrom(stats,update)
importFrom(stats,xtabs)
importFrom(utils,combn)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,installed.packages)
importFrom(utils,packageDescription)
importFrom(utils,read.table)
importFrom(utils,stack)
importFrom(utils,write.table)
useDynLib(phangorn, .registration = TRUE)
